@@ -16,7 +16,7 @@ from pony.orm import Database, Required, db_session, Set, Optional, select, \
     PrimaryKey, RowNotFound, ERDiagramError, OperationalError
 
 from mutmut import BAD_TIMEOUT, OK_SUSPICIOUS, BAD_SURVIVED, UNTESTED, \
-    OK_KILLED, MutationID, Context, mutate
+    OK_KILLED, RelativeMutationID, Context, mutate
 
 db = Database()
 
@@ -127,7 +127,7 @@ def ranges(numbers):
         if start_range == end_range:
             result.append(str(start_range))
         else:
-            result.append(f'{start_range}-{end_range}')
+            result.append('{}-{}'.format(start_range, end_range))
 
     for x in numbers[1:]:
         if end_range + 1 == x:
@@ -182,7 +182,7 @@ def print_result_cache(show_diffs=False, dict_synonyms=None, print_only_filename
     print_stuff('Timed out ⏰', select(x for x in Mutant if x.status == BAD_TIMEOUT))
     print_stuff('Suspicious 🤔', select(x for x in Mutant if x.status == OK_SUSPICIOUS))
     print_stuff('Survived 🙁', select(x for x in Mutant if x.status == BAD_SURVIVED))
-    print_stuff('Untested', select(x for x in Mutant if x.status == UNTESTED))
+    print_stuff('Untested/skipped', select(x for x in Mutant if x.status == UNTESTED))
 
 
 def get_unified_diff(argument, dict_synonyms, update_cache=True, source=None):
@@ -292,7 +292,7 @@ def create_html_report(dict_synonyms):
                 def print_diffs(status):
                     mutants = mutants_by_status[status]
                     for mutant in sorted(mutants, key=lambda m: m.id):
-                        diff = _get_unified_diff(source, filename, MutationID(mutant.line.line, mutant.index, mutant.line.line_number), dict_synonyms, update_cache=False)
+                        diff = _get_unified_diff(source, filename, RelativeMutationID(mutant.line.line, mutant.index, mutant.line.line_number), dict_synonyms, update_cache=False)
                         f.write('<h3>Mutant %s</h3>' % mutant.id)
                         f.write('<pre>%s</pre>' % diff)
 
@@ -413,6 +413,41 @@ def update_mutant_status(file_to_mutate, mutation_id, status, tests_hash):
     mutant.tested_against_hash = tests_hash
 
 
+@init_db
+@db_session
+def get_cached_mutation_statuses(filename, mutations, hash_of_tests):
+    sourcefile = SourceFile.get(filename=filename)
+    assert sourcefile
+
+    line_obj_by_line = {}
+
+    result = {}
+
+    for mutation_id in mutations:
+        if mutation_id.line not in line_obj_by_line:
+            line_obj_by_line[mutation_id.line] = Line.get(sourcefile=sourcefile, line=mutation_id.line, line_number=mutation_id.line_number)
+        line = line_obj_by_line[mutation_id.line]
+        assert line
+        mutant = Mutant.get(line=line, index=mutation_id.index)
+        if mutant is None:
+            mutant = get_or_create(Mutant, line=line, index=mutation_id.index, defaults=dict(status=UNTESTED))
+
+        result[mutation_id] = mutant.status
+        if mutant.status == OK_KILLED:
+            # We assume that if a mutant was killed, a change to the test
+            # suite will mean it's still killed
+            result[mutation_id] = mutant.status
+        else:
+            if mutant.tested_against_hash != hash_of_tests or \
+                    mutant.tested_against_hash == NO_TESTS_FOUND or \
+                    hash_of_tests == NO_TESTS_FOUND:
+                result[mutation_id] = UNTESTED
+            else:
+                result[mutation_id] = mutant.status
+
+    return result
+
+
 @init_db
 @db_session
 def cached_mutation_status(filename, mutation_id, hash_of_tests):
@@ -441,12 +476,12 @@ def cached_mutation_status(filename, mutation_id, hash_of_tests):
 @db_session
 def mutation_id_from_pk(pk):
     mutant = Mutant.get(id=pk)
-    return MutationID(line=mutant.line.line, index=mutant.index, line_number=mutant.line.line_number)
+    return RelativeMutationID(line=mutant.line.line, index=mutant.index, line_number=mutant.line.line_number)
 
 
 @init_db
 @db_session
-def filename_and_mutation_id_from_pk(pk) -> Tuple[str, MutationID]:
+def filename_and_mutation_id_from_pk(pk) -> Tuple[str, RelativeMutationID]:
     mutant = Mutant.get(id=pk)
     if mutant is None:
         raise ValueError("Obtained null mutant for pk: {}".format(pk))
